=== modified file 'Makefile'
--- Makefile	2012-01-11 08:06:03 +0000
+++ Makefile	2012-01-22 10:39:15 +0000
@@ -1,4 +1,2 @@
 check:
-	python -m unittest grackle.tests.test_client
-
-.PHONY: check
+	python -m unittest grackle.tests.test_suite

=== added file 'grackle-create-instance'
--- grackle-create-instance	1970-01-01 00:00:00 +0000
+++ grackle-create-instance	2012-01-22 10:39:15 +0000
@@ -0,0 +1,23 @@
+#!/usr/bin/env python
+
+from optparse import OptionParser
+
+from grackle.model import create_schema
+
+
+parser = OptionParser(usage="usage: %prog [options] <hostname> <keyspace>")
+parser.add_option(
+    "--clobber", action="store_true", default=False, dest="clobber",
+    help="clear the keyspace before initialising it")
+parser.add_option(
+    "--create-keyspace", action="store_true", default=False,
+    dest="create_keyspace",
+    help="create the keyspace (with SimpleStrategy, RF=1)")
+options, args = parser.parse_args()
+
+if len(args) != 2:
+    parser.error("hostname and keyspace required")
+
+create_schema(
+    args[0], args[1], clobber=options.clobber,
+    create_keyspace=options.create_keyspace)

=== added file 'grackle/cassandra.py'
--- grackle/cassandra.py	1970-01-01 00:00:00 +0000
+++ grackle/cassandra.py	2012-01-22 10:39:15 +0000
@@ -0,0 +1,41 @@
+# Copyright (c) 2011 Canonical Ltd
+#
+# This program is free software: you can redistribute it and/or modify
+# it under the terms of the GNU Affero General Public License as published by
+# the Free Software Foundation, either version 3 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU Affero General Public License for more details.
+#
+# You should have received a copy of the GNU Affero General Public
+# License along with this program. If not, see
+# <http://www.gnu.org/licenses/>.
+
+"""Things to ease working with cassandra."""
+
+from pycassa.pool import InvalidRequestError
+
+
+def workaround_1779(callable, *args, **kwargs):
+    """Workaround cassandra not being able to do concurrent schema edits.
+
+    The callable is tried until it does not raised InvalidRequestException
+    with why = "Previous version mismatch. cannot apply."
+
+    :param callable: The callable to call.
+    :param args: The args for it.
+    :param kwargs: The kwargs for it.
+    :return: The result of calling the callable.
+    """
+    while True:
+        # Workaround https://issues.apache.org/jira/browse/CASSANDRA-1779:
+        # Cassandra cannot do concurrent schema changes.
+        try:
+            return callable(*args, **kwargs)
+            break
+        except InvalidRequestError as e:
+            if e.why != 'Previous version mismatch. cannot apply.':
+                raise

=== modified file 'grackle/client.py'
--- grackle/client.py	2012-01-11 14:04:41 +0000
+++ grackle/client.py	2012-01-20 08:26:17 +0000
@@ -1,50 +1,13 @@
 import httplib
-import simplejson
-from urlparse import urlunparse
-from urllib import (
-    quote,
-    urlencode,
-)
-
-
-
-class GrackleClient:
-
-    def __init__(self, host, port):
-        self.host = host
-        self.port = port
-        self.netloc = '%s:%d' % (host, port)
-
-    def archive_url(self, archive_name, query):
-        path = '/%s' % quote(archive_name)
-        query_string = urlencode(query)
-        return urlunparse(('http', self.netloc, path, '', query_string, ''))
-
-    def _get_connection(self):
-        return httplib.HTTPConnection(self.host, self.port)
-
-    def _verb_archive(self, verb, archive_name, query, body=None):
-        url = self.archive_url(archive_name, query)
-        connection = self._get_connection()
-        connection.request(verb, url, body)
-        return connection.getresponse()
-
-    def put_message(self, archive_name, key, file_obj):
-        response = self._verb_archive(
-            'POST', archive_name, {'key': key}, file_obj.read())
-        data = response.read()
-        if response.status == httplib.BAD_REQUEST:
-            raise Exception('wtf')
-        elif response.status == httplib.CREATED:
-            return
-        else:
-            raise Exception('!!')
-
-    def get_messages(self, archive_name, message_ids=None):
-        parameters = {}
-        if message_ids is not None:
-            parameters['message_ids'] = message_ids
-        query = {'parameters': simplejson.dumps(parameters)}
-        response = self._verb_archive('GET', archive_name, query)
-        data = response.read()
-        return simplejson.loads(data)
+
+def put_message(archive_name, file_obj):
+    connection = httplib.HTTPConnection('localhost', 8435)
+    connection.request('POST', '/archive/%s' % archive_name, file_obj.read())
+    response = connection.getresponse()
+    data = response.read()
+    if response.status == httplib.BAD_REQUEST:
+        raise Exception('wtf')
+    elif response.status == httplib.CREATED:
+        return
+    else:
+        raise Exception('!!')

=== added file 'grackle/model.py'
--- grackle/model.py	1970-01-01 00:00:00 +0000
+++ grackle/model.py	2012-01-22 12:35:09 +0000
@@ -0,0 +1,151 @@
+# Copyright (c) 2012 Canonical Ltd
+#
+# This program is free software: you can redistribute it and/or modify
+# it under the terms of the GNU Affero General Public License as published by
+# the Free Software Foundation, either version 3 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU Affero General Public License for more details.
+#
+# You should have received a copy of the GNU Affero General Public
+# License along with this program. If not, see
+# <http://www.gnu.org/licenses/>.
+
+import datetime
+import dateutil.tz
+import email.parser
+from email.utils import parsedate_tz
+import logging
+import uuid
+
+import pycassa
+from pycassa.system_manager import (
+    LEXICAL_UUID_TYPE,
+    SystemManager,
+    TIME_UUID_TYPE,
+    )
+
+from grackle.cassandra import workaround_1779
+
+
+def create_schema(host, keyspace, clobber=False, create_keyspace=False):
+    mgr = SystemManager(host)
+
+    if create_keyspace:
+        mgr.create_keyspace(keyspace, replication_factor=1)
+
+    if clobber:
+        for cf in mgr.get_keyspace_column_families(keyspace):
+            mgr.drop_column_family(keyspace, cf)
+
+    try:
+        workaround_1779(
+            mgr.create_column_family, keyspace, 'message',
+            key_validation_class=LEXICAL_UUID_TYPE)
+        workaround_1779(
+            mgr.create_column_family, keyspace, 'archive_message',
+            comparator_type=TIME_UUID_TYPE,
+            default_validation_class=LEXICAL_UUID_TYPE)
+        pass
+    finally:
+        mgr.close()
+
+
+def _parse_message(message):
+    """Get a date and dict of an RFC822 message."""
+    parsed = email.parser.Parser().parsestr(message)
+    message_dict = {}
+
+    for key in ('from', 'to', 'subject', 'message-id'):
+        value = parsed.get(key, None)
+        if value is not None:
+            message_dict[key] = value
+
+    date = parsed.get('date')
+    if date is not None:
+        try:
+            pdate = parsedate_tz(date)
+            date = datetime.datetime(
+                *pdate[:6],
+                tzinfo=dateutil.tz.tzoffset('', pdate[9]))
+        except ValueError:
+            pass
+    message_dict['date'] = date.isoformat() if date is not None else None
+
+    return date, message_dict
+
+
+class CassandraConnection(object):
+
+    def __init__(self, keyspace, host):
+        self._keyspace = keyspace
+        self._host = host
+        self._connection = self._connect()
+        self.messages = self._column_family('message')
+        self.archive_messages = self._column_family('archive_message')
+
+    def _connect(self):
+        return pycassa.connect(self._keyspace, self._host)
+
+    def _column_family(self, name):
+        return pycassa.ColumnFamily(self._connection, name)
+
+    def add_message(self, archive_uuid, message):
+        message_uuid = uuid.uuid4()
+        message_date, message_dict = _parse_message(message)
+        message_dict['content'] = message
+        message_dict['date_created'] = (
+            datetime.datetime.utcnow().isoformat() + 'Z')
+        self.messages.insert(message_uuid, message_dict)
+        self.archive_messages.insert(
+            archive_uuid,
+            {message_date.astimezone(dateutil.tz.tzutc()): message_uuid})
+        logging.debug(
+            'Imported %s into %s'
+            % (message_dict.get('message-id', None), archive_uuid))
+        return message_uuid
+
+    def _format_message(self, message):
+        return {
+            'date': message.get('date'),
+            'from': message.get('from'),
+            'subject': message.get('subject'),
+            'message-id': message.get('message-id'),
+            }
+
+    def get_messages(self, archive_uuid, order, count, memo):
+        if order in ("date", "-date"):
+            reversed = order[0] == '-'
+        else:
+            raise AssertionError("Unsupported order.")
+        if memo != '':
+            memo = uuid.UUID(memo)
+        # Get up to n+1 messages from the memo: the last item of the
+        # previous batch (because that's where the memo starts) + this
+        # batch.
+        pairs = self.archive_messages.get(
+            archive_uuid, column_count=count + 1,
+            column_start=memo, column_reversed=reversed).items()
+
+        if memo and len(pairs) and pairs[0][0] <= memo:
+            pairs = pairs[1:]
+        elif len(pairs) > count:
+            pairs = pairs[:-1]
+
+        if len(pairs) == 0:
+            return (None, [], None)
+
+        assert 0 < len(pairs) <= count
+
+        ids = [v for k, v in pairs]
+        messages = self.messages.multiget(
+            ids, columns=['date', 'from', 'subject', 'message-id'])
+
+        return (
+            str(pairs[0][0]),
+            [self._format_message(messages[id]) for id in ids],
+            str(pairs[-1][0]),
+            )

=== added directory 'grackle/testing'
=== added file 'grackle/testing/__init__.py'
=== added file 'grackle/testing/cassandra.py'
--- grackle/testing/cassandra.py	1970-01-01 00:00:00 +0000
+++ grackle/testing/cassandra.py	2012-01-22 10:39:15 +0000
@@ -0,0 +1,54 @@
+# Copyright (c) 2011 Canonical Ltd
+#
+# This program is free software: you can redistribute it and/or modify
+# it under the terms of the GNU Affero General Public License as published by
+# the Free Software Foundation, either version 3 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU Affero General Public License for more details.
+#
+# You should have received a copy of the GNU Affero General Public
+# License along with this program. If not, see
+# <http://www.gnu.org/licenses/>.
+
+"""Test helpers for working with cassandra."""
+
+import os.path
+
+from fixtures import Fixture, TempDir
+from pycassa.system_manager import SystemManager
+
+from grackle.cassandra import workaround_1779
+from grackle.model import create_schema
+
+
+class TemporaryKeyspace(Fixture):
+    """Create a temporary keyspace.
+
+    The keyspace is named after a tempdir.
+    """
+
+    def setUp(self):
+        super(TemporaryKeyspace, self).setUp()
+        tempdir = self.useFixture(TempDir())
+        self.keyspace = os.path.basename(tempdir.path)
+        self.mgr = SystemManager()
+        workaround_1779(self.mgr.create_keyspace, self.keyspace,
+            replication_factor=1)
+        self.addCleanup(workaround_1779, self.mgr.drop_keyspace,
+            self.keyspace)
+
+
+class TemporaryDB(Fixture):
+    """Create a temporary usable OOPS DB.
+
+    The keyspace for it is at self.keyspace.
+    """
+
+    def setUp(self):
+        super(TemporaryDB, self).setUp()
+        self.keyspace = self.useFixture(TemporaryKeyspace()).keyspace
+        create_schema('localhost', self.keyspace)

=== modified file 'grackle/tests/__init__.py'
--- grackle/tests/__init__.py	2012-01-10 11:16:56 +0000
+++ grackle/tests/__init__.py	2012-01-22 10:39:15 +0000
@@ -0,0 +1,28 @@
+# Copyright (c) 2012 Canonical Ltd
+#
+# This program is free software: you can redistribute it and/or modify
+# it under the terms of the GNU Affero General Public License as published by
+# the Free Software Foundation, either version 3 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU Affero General Public License for more details.
+#
+# You should have received a copy of the GNU Affero General Public
+# License along with this program. If not, see
+# <http://www.gnu.org/licenses/>.
+
+from unittest import TestLoader
+
+
+def test_suite():
+    test_names = [
+        'model',
+        'client',
+        'wsgi',
+        ]
+    tests = ['grackle.tests.test_' + test for test in test_names]
+    loader = TestLoader()
+    return loader.loadTestsFromNames(tests)

=== modified file 'grackle/tests/test_client.py'
--- grackle/tests/test_client.py	2012-01-11 14:04:41 +0000
+++ grackle/tests/test_client.py	2012-01-10 14:03:53 +0000
@@ -5,43 +5,27 @@
 import httplib
 import os
 from signal import SIGKILL
-import simplejson
 from StringIO import StringIO
 from unittest import TestCase
-from urlparse import urlparse
-from urlparse import parse_qs
 
 from testtools import ExpectedException
 
-from grackle.client import (
-    GrackleClient,
-    )
-
-
-class ForkedFake:
-
-    def __init__(self, port, messages=None):
+from grackle import client
+
+
+class Forked:
+
+    def __init__(self, func_or_method):
+        self.func_or_method = func_or_method
         self.pid = None
-        self.port = port
-        self.messages = messages
-        self.read_end, self.write_end = os.pipe()
-
-    def is_ready(self):
-        os.write(self.write_end, 'asdf')
 
     def __enter__(self):
         pid = os.fork()
-        if pid == 0:
-            self.start_server()
-        self.pid = pid
-        os.read(self.read_end, 1)
-        return
+        if pid != 0:
+            self.pid = pid
+            return
+        self.func_or_method()
 
-    def start_server(self):
-        service = HTTPServer(('', self.port), FakeGrackleRequestHandler)
-        service.messages = self.messages
-        self.is_ready()
-        service.serve_forever()
 
     def __exit__(self, exc_type, exc_val, traceback):
         os.kill(self.pid, SIGKILL)
@@ -58,54 +42,17 @@
         else:
             self.send_error(httplib.BAD_REQUEST)
 
-    def do_GET(self):
-        scheme, netloc, path, params, query_string, fragments = (
-            urlparse(self.path))
-        archive = os.path.split(path)[1]
-        query = parse_qs(query_string)
-        parameters = simplejson.loads(query['parameters'][0])
-        self.send_response(httplib.OK)
-        self.end_headers()
-        messages = [m for m in self.server.messages[archive] if 'message_ids'
-                    not in parameters or m['message-id'] in
-                    parameters['message_ids']]
-        self.wfile.write(simplejson.dumps(messages))
-
-
-def fake_grackle_service(client, messages=None):
-    if messages is None:
-        messages = {}
-    return ForkedFake(client.port, messages)
+
+def run_service():
+    service = HTTPServer(('', 8435), FakeGrackleRequestHandler)
+    service.serve_forever()
+
 
 
 class TestPutMessage(TestCase):
 
     def test_put_message(self):
-        client = GrackleClient('localhost', 8436)
-        with fake_grackle_service(client):
-            client.put_message('arch1', 'asdf', StringIO('This is a message'))
+        with Forked(run_service):
+            client.put_message('arch1', StringIO('This is a message'))
             with ExpectedException(Exception, 'wtf'):
-                client.put_message('arch1', 'asdf',
-                    StringIO('This is not a message'))
-
-
-class TestGetMessages(TestCase):
-
-    def test_get_messages(self):
-        client = GrackleClient('localhost', 8435)
-        with fake_grackle_service(client,
-            {'baz':
-            [{'message-id': 'foo'},
-             {'message-id': 'bar'}]}):
-            response = client.get_messages('baz')
-        self.assertEqual(['bar', 'foo'], sorted(m['message-id'] for m in
-            response))
-
-    def test_get_messages_by_id(self):
-        client = GrackleClient('localhost', 8437)
-        with fake_grackle_service(client,
-            {'baz':
-            [{'message-id': 'foo'},
-             {'message-id': 'bar'}]}):
-            message, = client.get_messages('baz', message_ids=['foo'])
-        self.assertEqual('foo', message['message-id'])
+                client.put_message('arch1', StringIO('This is not a message'))

=== added file 'grackle/tests/test_model.py'
--- grackle/tests/test_model.py	1970-01-01 00:00:00 +0000
+++ grackle/tests/test_model.py	2012-01-22 12:35:09 +0000
@@ -0,0 +1,169 @@
+# Copyright (c) 2012 Canonical Ltd
+#
+# This program is free software: you can redistribute it and/or modify
+# it under the terms of the GNU Affero General Public License as published by
+# the Free Software Foundation, either version 3 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU Affero General Public License for more details.
+#
+# You should have received a copy of the GNU Affero General Public
+# License along with this program. If not, see
+# <http://www.gnu.org/licenses/>.
+
+__metaclass__ = type
+
+import datetime
+from email.utils import formatdate
+import os
+from tempfile import _RandomNameSequence
+import time
+import unittest
+
+from dateutil.tz import (
+    tzoffset,
+    tzutc,
+    )
+from pycassa.util import convert_uuid_to_time
+
+from grackle.model import (
+    CassandraConnection,
+    _parse_message,
+    )
+
+
+TEMPLATE_MESSAGE = """\
+From: sysadmin@example.com
+To: developer@example.com
+Subject: Everything is broken
+Date: {date}
+Message-Id: {id}
+
+Help, everything has just broken.
+"""
+
+TEST_MESSAGE = TEMPLATE_MESSAGE.format(
+    date='Sat, 1 Jan 2000 11:02:34 +1100',
+    id='<aaaaaaaaaaaaa@example.com>')
+
+
+class TestParseMessage(unittest.TestCase):
+
+    def test_works(self):
+        # _parse_message extracts interesting fields. It also parses the
+        # date and returns it separately.
+        date, msg = _parse_message(TEST_MESSAGE)
+        self.assertEqual('sysadmin@example.com', msg['from'])
+        self.assertEqual('developer@example.com', msg['to'])
+        self.assertEqual('Everything is broken', msg['subject'])
+        self.assertEqual('2000-01-01T11:02:34+11:00', msg['date'])
+        self.assertEqual('<aaaaaaaaaaaaa@example.com>', msg['message-id'])
+        self.assertEqual(
+            datetime.datetime(
+                2000, 1, 1, 11, 2, 34, tzinfo=tzoffset('', 39600)),
+            date)
+
+
+class TestAddMessage(unittest.TestCase):
+
+    def test_add_message(self):
+        c = CassandraConnection(
+            os.environ['GRACKLE_TEST_KEYSPACE'], ['localhost:9160'])
+        archive = next(_RandomNameSequence())
+
+        # Write the message out to Cassandra, and read it back in.
+        key = c.add_message(archive, TEST_MESSAGE)
+        cmsg = c.messages.get(key)
+
+        # The archive should contain a single message, a reference to
+        # our new key.
+        archive_messages = c.archive_messages.get(archive).items()
+        self.assertEqual(1, len(archive_messages))
+        self.assertEqual(key, archive_messages[0][1])
+
+        # The key in archive_message is a TimeUUID for the Date field in
+        # the message.
+        utctime = time.mktime(datetime.datetime(
+            2000, 1, 1, 0, 2, 34, tzinfo=tzutc()).timetuple())
+        self.assertEqual(
+            utctime,
+            convert_uuid_to_time(archive_messages[0][0]))
+
+        # The stored message contains the full original text of the
+        # message, as well as interesting fields parsed out.
+        self.assertEqual(TEST_MESSAGE, cmsg['content'])
+        parsed_message = _parse_message(TEST_MESSAGE)[1]
+        for key, value in parsed_message.iteritems():
+            self.assertEqual(value, cmsg[key])
+
+
+class TestGetMessages(unittest.TestCase):
+
+    def assertMessages(self, expected_ids, messages):
+        expected_msgids = [
+            '<message%d@example.com>' % id for id in expected_ids]
+        actual_msgids = [msg['message-id'] for msg in messages]
+        self.assertEqual(expected_msgids, actual_msgids)
+
+    def makeMessages(self, conn, archive, count):
+        return [
+            conn.add_message(
+                archive,
+                TEMPLATE_MESSAGE.format(
+                    date=formatdate(i * 100),
+                    id='<message%d@example.com>' % i))
+            for i in range(count)]
+
+    def makeArchive(self):
+        conn = CassandraConnection(
+            os.environ['GRACKLE_TEST_KEYSPACE'], ['localhost:9160'])
+        archive = next(_RandomNameSequence())
+        return conn, archive
+
+    def test_single_message(self):
+        conn, archive = self.makeArchive()
+        self.makeMessages(conn, archive, 1)
+        self.assertMessages(
+            [0], conn.get_messages(archive, 'date', 1, '')[1])
+
+    def test_limit(self):
+        conn, archive = self.makeArchive()
+        self.makeMessages(conn, archive, 4)
+        self.assertMessages(
+            [0, 1], conn.get_messages(archive, 'date', 2, '')[1])
+
+    def test_order(self):
+        conn, archive = self.makeArchive()
+        self.makeMessages(conn, archive, 4)
+        self.assertMessages(
+            [3, 2], conn.get_messages(archive, '-date', 2, '')[1])
+
+    def test_batching_forward(self):
+        conn, archive = self.makeArchive()
+        self.makeMessages(conn, archive, 5)
+        prev, messages, next = conn.get_messages(archive, 'date', 2, '')
+        self.assertMessages([0, 1], messages)
+        prev, messages, next = conn.get_messages(archive, 'date', 2, next)
+        self.assertMessages([2, 3], messages)
+        prev, messages, next = conn.get_messages(archive, 'date', 2, next)
+        self.assertMessages([4], messages)
+        prev, messages, next = conn.get_messages(archive, 'date', 2, next)
+        self.assertIs(None, prev)
+        self.assertMessages([], messages)
+        self.assertIs(None, next)
+
+    def test_batching_backward(self):
+        conn, archive = self.makeArchive()
+        self.makeMessages(conn, archive, 5)
+        prev, messages, next = conn.get_messages(archive, 'date', 2, '')
+        self.assertMessages([0, 1], messages)
+        prev, messages, next = conn.get_messages(archive, 'date', 2, next)
+        self.assertMessages([2, 3], messages)
+        # XXX: We shouldn't have to reverse the sort order. There should
+        # be a flag to work backwards, but I'm not sure if we can get
+        # that from Cassandra without reversing in Python.
+        prev, messages, next = conn.get_messages(archive, '-date', 2, prev)
+        self.assertMessages([1, 0], messages)

=== added file 'grackle/tests/test_wsgi.py'
--- grackle/tests/test_wsgi.py	1970-01-01 00:00:00 +0000
+++ grackle/tests/test_wsgi.py	2012-01-22 10:39:15 +0000
@@ -0,0 +1,41 @@
+# Copyright (c) 2012 Canonical Ltd
+#
+# This program is free software: you can redistribute it and/or modify
+# it under the terms of the GNU Affero General Public License as published by
+# the Free Software Foundation, either version 3 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU Affero General Public License for more details.
+#
+# You should have received a copy of the GNU Affero General Public
+# License along with this program. If not, see
+# <http://www.gnu.org/licenses/>.
+
+__metaclass__ = type
+
+from unittest import TestCase
+
+from webtest import TestApp
+
+from grackle.wsgi import app
+
+
+class TestRoot(TestCase):
+
+    def test_get(self):
+        test_app = TestApp(app.wsgifunc())
+        r = test_app.get('/')
+        self.assertEqual(200, r.status_int)
+        r.mustcontain('Service endpoint for Grackle.')
+
+
+class TestArchive(TestCase):
+
+    def test_get(self):
+        test_app = TestApp(app.wsgifunc())
+        r = test_app.get('/archive/foo')
+        self.assertEqual(200, r.status_int)
+        r.mustcontain('Service for archive foo.')

=== added file 'grackle/wsgi.py'
--- grackle/wsgi.py	1970-01-01 00:00:00 +0000
+++ grackle/wsgi.py	2012-01-22 12:36:48 +0000
@@ -0,0 +1,86 @@
+# Copyright (c) 2011 Canonical Ltd
+#
+# This program is free software: you can redistribute it and/or modify
+# it under the terms of the GNU Affero General Public License as published by
+# the Free Software Foundation, either version 3 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU Affero General Public License for more details.
+#
+# You should have received a copy of the GNU Affero General Public
+# License along with this program. If not, see
+# <http://www.gnu.org/licenses/>.
+
+__metaclass__ = type
+
+import json
+import uuid
+
+import web
+
+from grackle.model import CassandraConnection
+
+conn = None
+
+
+class root:
+
+    def GET(self):
+        web.header('Content-Type', 'text/plain')
+        return (
+            'Service endpoint for Grackle.\n\n'
+            'When you want mail that was archived in Grackle, you can get it '
+            'from Grackle')
+
+
+class archive:
+
+    def GET(self, name):
+        web.header('Content-Type', 'text/plain')
+        args = web.input()
+        if not getattr(args, 'op', None):
+            return self.index(name)
+
+        if args.op == 'get_messages':
+            return self.get_messages(name, args)
+
+    def POST(self, name):
+        data = web.data()
+        conn.add_message(name, data)
+        web.ctx.status = '201 Created'
+
+    def index(self, name):
+        return ('Service for archive %s.' % name)
+
+    def get_messages(self, name, args):
+        try:
+            count = int(args.count)
+        except (AttributeError, ValueError):
+            count = 10
+        order = getattr(args, 'order', '-date')
+        memo = getattr(args, 'memo', '')
+        prev, messages, next = conn.get_messages(name, order, count, memo)
+        return json.dumps({
+            'prev_memo': prev,
+            'messages': messages,
+            'next_memo': next,
+            })
+
+
+urls = (
+    '/', 'root',
+    '/archive/(.+)', 'archive',
+    )
+
+app = web.application(urls, globals(), autoreload=False)
+
+if __name__ == "__main__":
+    import logging
+    logging.basicConfig(
+        format='%(asctime)s:%(levelname)s:%(name)s:%(message)s',
+        level=logging.DEBUG)
+    conn = CassandraConnection('grackle', ['localhost:9160'])
+    app.run()

